library(magrittr)
library(tidyverse)
library(stringr)
library(readxl)
library(writexl)

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()

# Before spring 2022
d_csv <- list.files(pattern = "\\.csv$", recursive = TRUE) %>% 
  as_tibble_col("path") %>% 
  mutate(id = str_extract(path, "\\d{8}"),
         file = ifelse(str_detect(path, "hepsiburada"), "test", "train"),
         content = map(path, read_csv, col_types = "c", comment = "#")) %>% 
  unnest(content) %>% 
  select(path:RANK) %>% 
  filter(id != "21401131") %>% 
  # Update laptop measures
  mutate(Alternative = as.character(Alternative),
         Memory = ifelse(Memory == "4 GB", "8 GB", "16 GB"),
         HardDisk = ifelse(HardDisk == "500 GB", "750 GB", "1.000 GB"),
         # Price = scales::comma(parse_number(Price)*4, big.mark = ".", decimal.mark = ",")
         Price = case_when(Price == "1500 TL" ~ "6.000 TL",
                           Price == "1700 TL" ~ "7.000 TL",
                           Price == "2000 TL" ~ "8.500 TL",
                           Price == "3500 TL" ~ "10.000 TL")
  # mutate(Memory = sprintf("% 5s", ifelse(Memory == "4 GB", "8 GB", "16 GB")),
  #        HardDisk = sprintf("% 6s", ifelse(HardDisk == "500 GB", "750 GB", "1 TB")),
  #        # Price = sprintf("% 6s TL", scales::comma(parse_number(Price)*4, big.mark = ".", decimal.mark = ","))
  #        Price = sprintf("% 9s", case_when(Price == "1500 TL" ~ "6.000 TL",
  #                                          Price == "1700 TL" ~ "7.000 TL",
  #                                          Price == "2000 TL" ~ "8.500 TL",
  #                                          Price == "3500 TL" ~ "10.000 TL"))
  )

# Are there any questionnaire short of 16 items in train 6 items in test?
d_csv %>% 
  distinct(file, Alternative, Brand) %>% 
  print(n=Inf)

# After spring 2022
d_xlsx <- list.files(pattern = "\\.xlsx$", recursive = TRUE) %>% 
  discard(~str_detect(., "survey results")) %>% 
  as_tibble_col("path") %>% 
  filter(str_detect(path, "Roster", negate = TRUE)) %>% 
  mutate(id = str_extract(path, "\\d{8}"),
         file = ifelse(str_detect(path, "hepsiburada"), "test", "train"),
         content = map(path, read_excel)) %>% 
         #content = map(path, ~{read_excel(.x) %>% mutate(Alternative = as.character(Alternative))})) %>% 
  unnest(content) %>% 
  select(path:RANK) %>% 
  rename(Speed = `Speed (GHz)`,
         Memory = `Memory (GB)`,
         HardDisk = `HardDisk (GB)`,
         Screen = `Screen (in)`,
         Price = `Price (TL)`) %>% 
  mutate(Alternative = as.character(Alternative),
         Speed = paste(Speed, "GHz"),
         Memory = paste(Memory, "GB"),
         HardDisk = case_when(HardDisk == 1000 ~ "1.000 GB",
                              HardDisk == 750  ~ "750 GB"),
         Screen = paste(Screen, "in"),
         Price = paste(scales::comma(Price, big.mark = ".", decimal.mark = ","), "TL"))

d_xlsx %>% 
  distinct(file, Alternative, Brand) %>% 
  print(n=Inf)


# Is there a mismatch between alternatives of csv and xlsx files?
# design_csv <- d_csv %>%
#   filter(file == "train") %>%
#   distinct(Alternative, Brand, Processor, Memory, HardDisk, Screen, Price) %>%
#   rename(Alternative_csv = Alternative)
# 
# design_xslx <- d_xlsx %>% 
#   filter(file == "train") %>% 
#   distinct(Alternative, Brand, Processor, Memory, HardDisk, Screen, Price) %>% 
#   rename(Alternative_xlsx = Alternative)
# 
# aa <- design_xslx %>% 
#   left_join(design_csv) %>% 
#   pull(Alternative_csv)


d <- list(d_csv, d_xlsx) %>% 
  bind_rows()

# codify the data
d_coded <- d %>% 
  mutate(semester = str_extract(path, "^.*?(?= )"),
         year = str_extract(path, "\\d*(?=/)")) %>% 
  nest(data = -c(id, semester, year)) %>% 
  # distinct(id) %>% 
  # count()
  rename(idreal = id) %>% 
  arrange(idreal) %>% 
  mutate(cohort = str_extract(idreal, "^.{3}")) %>% 
  group_by(cohort) %>% 
  mutate(id = sprintf("y%s%s%ss%03d", str_sub(year, 3, 4), str_sub(str_to_lower(semester), 1, 1), cohort, seq(n()))) %>% 
  ungroup() %>% 
  select(-idreal, -cohort, -semester, -year) %>% 
  relocate(id) %>% 
  unnest(data) %>% 
  select(-path) %>% 
  arrange(id, file, as.numeric(Alternative))

# d_coded %>% filter(id == "f16c212s003") %>% print(n=Inf)

d_coded %>% 
  distinct(file, Alternative, Brand) %>% 
  print(n=Inf)

d_coded %>% 
  filter(file == "test") %>% 
  select(-file) %>% 
  write_xlsx(path = "laptops test survey results-all-coded.xlsx")

d_coded %>% 
  filter(file == "train") %>% 
  select(-file) %>% 
  write_xlsx(path = "laptops train survey results-all-coded.xlsx")  
